import json
import os

from lemoji.generator.web_extractor import WebExtractor
from lemoji.data.tree import tree


class Generator:
    def __init__(self):
        self.tree = WebExtractor.build()
        self.working_dir = os.path.join(os.getcwd(), 'lemoji')
        self.data_file = os.path.join(os.getcwd(), 'lemoji', 'data', 'tree.py')

    @staticmethod
    def find_node(node_list, name):
        for node in node_list:
            if node['name'] == name:
                return node

    def generate(self, his_nodes=None, cur_nodes=None, version=None):
        if not version:
            his_nodes = tree
            cur_nodes = self.tree
        else:
            version = version.replace('.', '_')

        emoji_list = []

        for node in cur_nodes:
            his_node = self.find_node(his_nodes, node['name'])
            if node['type'] == 'folder':
                if not his_node:
                    self.generate([], node['children'], version=node['name'])
                elif node['time'] != his_node['time']:
                    self.generate(his_node['children'], node['children'], version=node['name'])
            elif node['name'].startswith('emoji'):
                emoji_list.extend(WebExtractor.emoji_fetcher(node['link']))

        if version:
            path = os.path.join(self.working_dir, 'emoji_v%s.py' % version)
            emoji_list = list(set(emoji_list))
            with open(path, 'wb+') as f:
                s = 'EMOJI_LIST = ' + json.dumps(emoji_list, ensure_ascii=False)
                f.write(s.encode())
        else:
            with open(self.data_file, 'wb+') as f:
                s = 'tree = ' + json.dumps(cur_nodes, ensure_ascii=False) + '\n'
                f.write(s.encode())
            with open(os.path.join(self.working_dir, '__init__.py'), 'wb+') as f:
                for node in cur_nodes:
                    name = node['name'].replace('.', '_')
                    s = 'from .emoji_v%s import EMOJI_LIST as EMOJI_LIST_%s\n' % (
                        name, name.upper())
                    f.write(s.encode())

                s = '\n\nEMOJI_LIST = EMOJI_LIST_%s\n' % \
                    cur_nodes[-1]['name'].replace('.', '_').upper()
                f.write(s.encode())



import requests

from bs4 import BeautifulSoup as Souper, Tag

proxies = {
  "http": "http://localhost:1087",
  "https": "http://localhost:1087",
}


class WebExtractor:
    base_url = 'https://www.unicode.org/Public/emoji/'

    @staticmethod
    def emoji_fetcher(url):
        print('Retrieving', url, '...')
        r = requests.get(url, proxies=proxies)
        data = r.content.decode()
        r.close()

        lines = list(filter(lambda line_: not line_.startswith('#'), data.split('\n')))
        lines = list(filter(lambda line_: line_.find(';') >= 0, lines))
        lines = list(map(lambda line_: line_[:line_.find(';') - 1].strip(), lines))

        emoji_list = []
        for line in lines:
            seq_mark = line.find('..')
            if seq_mark > 0:
                left = int(line[:seq_mark], 16)
                right = int(line[seq_mark+2:], 16)
                emoji_list.extend(map(chr, range(left, right + 1)))
            else:
                emoji_list.append(''.join(map(lambda s: chr(int(s, 16)), line.split(' '))))

        return emoji_list

    @classmethod
    def analyse(cls, url):
        r = requests.get(url, proxies=proxies)
        soup = Souper(r.content.decode(), 'html.parser')
        r.close()

        img_list = soup.find_all('img')
        node_list = []
        for img in img_list:  # type: Tag
            if img.get('src') == '/icons/folder.gif':
                type_ = 'folder'
            elif img.get('src') == '/icons/text.gif':
                type_ = 'file'
            else:
                continue
            a_tag = img.next_element.next_element  # type: Tag
            node = dict(
                type=type_,
                link=url + a_tag.get('href'),
                name=a_tag.text,
            )
            if node['name'].endswith('/'):
                node['name'] = node['name'][:-1]

            time = a_tag.next_sibling  # type: str
            time = time.strip()
            time = time[:time.rfind(' ')].strip()
            node['time'] = time
            node_list.append(node)

        return node_list

    @classmethod
    def build(cls, url=None):
        url = url or cls.base_url
        print('Retrieving', url, '...')
        node_list = cls.analyse(url)
        for node in node_list:
            if node['type'] == 'folder':
                node['children'] = cls.build(node['link'])
        return node_list



from .emoji_v1_0 import EMOJI_LIST as EMOJI_LIST_1_0
from .emoji_v2_0 import EMOJI_LIST as EMOJI_LIST_2_0
from .emoji_v3_0 import EMOJI_LIST as EMOJI_LIST_3_0
from .emoji_v4_0 import EMOJI_LIST as EMOJI_LIST_4_0
from .emoji_v5_0 import EMOJI_LIST as EMOJI_LIST_5_0
from .emoji_v11_0 import EMOJI_LIST as EMOJI_LIST_11_0
from .emoji_v12_0 import EMOJI_LIST as EMOJI_LIST_12_0
from .emoji_v12_1 import EMOJI_LIST as EMOJI_LIST_12_1
from .emoji_v13_0 import EMOJI_LIST as EMOJI_LIST_13_0


EMOJI_LIST = EMOJI_LIST_13_0



from setuptools import setup, find_packages

setup(
    name='lEmoji',
    version='0.1 beta',
    keywords=('emoji',),
    description='emoji list for all version based on unicode.org',
    license='MIT Licence',
    url='https://github.com/lqj679ssn/emoji',
    author='Jyonn Liu',
    author_email='i@6-79.cn',
    platforms='any',
    packages=find_packages(),
    install_requires=[],
)
